import { Component, Input } from "@angular/core";
import { FormBuilder, FormGroup, Validators } from "@angular/forms";
import { NgbActiveModal } from "@ng-bootstrap/ng-bootstrap";
import { BuildingService } from "../../../service/monitoring-api.service";
import { LoginService } from "../../../service/login.service";

@Component({
  selector: "app-add-edit-master-building",
  templateUrl: "./add-edit-master-building.component.html",
  styleUrls: ["./add-edit-master-building.component.css"],
})
export class AddEditMasterBuildingComponent {
  @Input() dataRow: any;
  @Input() mode: any;
  myForm: FormGroup;

  data: any;
  dataStatus: any[];
  filteredDataStatus: any[];
  dataMasterStatus: any;
  labelModal: any;
  constructor(
    public activeModal: NgbActiveModal,
    private fb: FormBuilder,
    private monitoringApiService: BuildingService,
    private authService: LoginService
  ) {}

  ngOnInit() {
    this.authService.startTokenCheck();
    this.authService.startTrackingActivity();
    this.createForm();
    this.listDataStatus();
    if (this.mode === "add") {
      this.labelModal = "Add Row"
    } else {
      this.labelModal = "Edit Row"
    }
    if (this.dataRow) {
      this.editForm();
    }
  }

  createForm() {
    this.myForm = this.fb.group({
      name: ["", Validators.required],
      statusId: ["", Validators.required],
      email: ["", [Validators.required, Validators.pattern(/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/)]],
      owner: ["", Validators.required],
      address: ["", Validators.required],
      phone: ["", [Validators.required, Validators.pattern(/^[0-9]{10,15}$/)]],
      kwh: ["", [Validators.required, Validators.maxLength(15), Validators.pattern(/^[0-9]{1,7}$/)]],
    });
  }

  editForm() {
    this.myForm = this.fb.group({
      id: this.dataRow.id,
      name: [this.dataRow.name, Validators.required],
      statusId: [this.dataRow.status_id, Validators.required],
      email: [this.dataRow.email, [Validators.required, Validators.pattern(/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/)]],
      address: [this.dataRow.address, Validators.required],
      phone: [this.dataRow.phone, [Validators.required, Validators.pattern(/^[0-9]{10,15}$/)]],
      owner: [this.dataRow.owner,Validators.required],
      kwh: [this.dataRow.kwh, [Validators.required, Validators.maxLength(15), Validators.pattern(/^[0-9]{1,7}$/)]],
    });
  }

  listDataStatus() {
    this.monitoringApiService.getMasterData().subscribe((res) => {
      this.data = res.results.data;
      this.dataMasterStatus = res.results.data.find(
        (item) => item.name === "master_status"
      );
      this.dataStatus = this.dataMasterStatus.headerDetailParam;
      console.log(this.dataStatus);

      this.filteredDataStatus = this.dataStatus.filter(
        (item) => item.statusName.toLowerCase() === "aktif"
      );
    });
  }

  addRow() {
    if (this.myForm.valid) {
      this.activeModal.close(this.myForm.value);
    } else {
      this.markFormGroupTouched(this.myForm);
    }
  }

  markFormGroupTouched(formGroup: FormGroup) {
    (Object as any).values(formGroup.controls).forEach((control) => {
      control.markAsTouched();
      if (control.controls) {
        this.markFormGroupTouched(control);
      }
    });
  }

  addDepartment() {
    let newDept = prompt("Enter new department name:");
    if (newDept) {
      alert(`Department ${newDept} added! (simulated)`);
    }
  }
}
