﻿import { Component, OnInit, Renderer2 } from "@angular/core";
import { Router, ActivatedRoute } from "@angular/router";
import { FormBuilder, FormGroup, Validators } from "@angular/forms";
import { AuthService } from "../_services/auth.service";
import { AlertService } from "../_services/alert.service";
import { HttpClient } from "@angular/common/http";
import { BuildingService } from "../content/hemat-app/service/monitoring-api.service";
import { ToastrService } from "ngx-toastr";
import { jwtDecode } from "jwt-decode";

interface CustomJwtPayload {
  exp: number;
  scope: string;
  iat: number;
  preferred_username: string;
  name: string;
  email: string;
  family_name: string;
  given_name: string;
  sub: string;
}

@Component({
  templateUrl: "login.component.html",
  styleUrls: ["./login.component.css"],
})
export class LoginComponent implements OnInit {
  loginForm: FormGroup;
  loading = false;
  submitted = false;
  returnUrl: string;
  isPageLoaded = false;

  constructor(
    private formBuilder: FormBuilder,
    private route: ActivatedRoute,
    public authService: AuthService,
    private renderer: Renderer2,
    private router: Router,
    private monitoringApiService: BuildingService,
    private toastr: ToastrService
  ) {
    this.route.queryParams.subscribe((params) => {
      this.returnUrl = params["returnUrl"];
    });
  }

  ngOnInit() {
    this.loginForm = this.formBuilder.group({
      email: ["", Validators.required],
      password: ["", Validators.required],
      rememberMe: false,
    });
    if (localStorage.getItem("currentUser")) {
      this.router.navigate(["/monitoring"]);
    }
  }

  get f() {
    return this.loginForm.controls;
  }

  tryLogin() {
    this.submitted = true;

    if (this.loginForm.invalid) {
      return;
    }

    const loginData = {
      username: this.loginForm.value.email,
      password: this.loginForm.value.password,
    };

    this.monitoringApiService.postLogin(loginData).subscribe(
      (response: any) => {
        const decodedToken = jwtDecode<CustomJwtPayload>(response.access_token);
        // localStorage.setItem("access_token", response.access_token);
        localStorage.setItem("account_info", JSON.stringify(decodedToken));

        const userProfile = {
          access_token: response.access_token,
          refresh_token: response.refresh_token,
          displayName: decodedToken.name,
          buildingId: 4,
        };
        this.toastr.success(
          "Success",
          "Selamat Datang " + userProfile.displayName,
          {
            timeOut: 5000,
            closeButton: true,
          }
        );
        localStorage.setItem("currentUser", JSON.stringify(userProfile));
        this.router.navigate(["/monitoring"]);
      },
      (error) => {
        this.submitted = false;
        this.toastr.error("Error", "Akun yang anda masukan salah!", {
          timeOut: 5000,
          closeButton: true,
        });
        console.error("Login failed:", error);
      }
    );
  }

  addCheckbox(event) {
    const toggle = document.getElementById("icheckbox");
    if (event.currentTarget.className === "icheckbox_square-blue") {
      this.renderer.addClass(toggle, "checked");
    } else if (
      event.currentTarget.className === "icheckbox_square-blue checked"
    ) {
      this.renderer.removeClass(toggle, "checked");
    }
  }

  setUserInStorage(res) {
    if (res.user) {
      localStorage.setItem("currentUser", JSON.stringify(res.user));
    } else {
      localStorage.setItem("currentUser", JSON.stringify(res));
    }
  }
}
